/*global Promise */

'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports['default'] = tokenStorage;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _utilsPreventRaceConditionJs = require('./utils/preventRaceCondition.js');

var _utilsPreventRaceConditionJs2 = _interopRequireDefault(_utilsPreventRaceConditionJs);

function tokenStorage(_ref) {
    var initialToken = _ref.initialToken;
    var fetchToken = _ref.fetchToken;
    var generateToken = _ref.generateToken;

    var _token = initialToken;

    var _fetchToken = fetchToken ? (0, _utilsPreventRaceConditionJs2['default'])(fetchToken) : function () {
        return Promise.reject(new Error('Getting a token from the server is not supported'));
    };
    var _generateToken = generateToken ? (0, _utilsPreventRaceConditionJs2['default'])(generateToken) : function () {
        return Promise.reject(new Error('Generating a token on the server is not supported'));
    };

    var getToken = function getToken() {
        if (_token) {
            return Promise.resolve(_token);
        }

        return _fetchToken().then(function (newToken) {
            return _token = newToken;
        })['catch'](_generateToken);
    };

    var refreshToken = function refreshToken() {
        _token = undefined;

        return _generateToken().then(function (newToken) {
            return _token = newToken;
        });
    };

    return {
        getToken: getToken,
        refreshToken: refreshToken
    };
}

module.exports = exports['default'];